/*******************************************************************************
 * Copyright (C) 2002, 2003
 * ingenieurbuero fuer innovative informationstechnik (iiit)
 * Dipl.-Ing. Joerg Beckmann, Dortmund, Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * version $Id: TcpServer.java,v 1.14 2003/04/14 20:41:41 joerg Exp $
 ******************************************************************************/

package de.iiit.access.server.plugins.server;

import de.iiit.access.common.api.*;

import de.iiit.access.server.*;
import de.iiit.access.server.api.*;
import de.iiit.access.server.plugins.parser.*;

import de.iiit.xmlconfig.*;
import de.iiit.util.*;

import java.util.regex.*;
import java.util.*;
import java.net.*;
import java.io.*;

import org.apache.log4j.Logger;

/** Implements a TCP/IP server for requesting the AccessServer 
 * @version $Revision: 1.14 $ $Date: 2003/04/14 20:41:41 $
 */
public class TcpServer extends Thread implements ThreadPluginIf
{
    /** CVS Version Tag */
    private static final String vcid = "$Id: TcpServer.java,v 1.14 2003/04/14 20:41:41 joerg Exp $";

    private static final String PORT = "Port";
    
    private static final int DEFAULT_PORT = 54321;
    
    private int port;

    private Logger logger = Logger.getLogger(this.getClass());

    private boolean running = true;
    
    private Vector handlers = new Vector();
    
    private ServerSocket listen = null;
    
    private boolean verifyUser = true;

    private ResolverPluginIf resolver = null;
    
    /** Creates a new instance of TcpServer */
    public TcpServer()
    {
        setName("TcpServer");

        logger.info("TcpServer intantiated.");
    }

    /** Initializes the server
     * @param config the configuration of the server. Currently the only parameter is <CODE>Port</CODE>
     * which defines the port number to listen on for connections. Default is <CODE>1234</CODE>
     */    
    public void initialize(Configuration config)
    {
        port = config.getIntAttribute(PORT, DEFAULT_PORT);
        
        verifyUser = AccessServer.getVerifyUser();        
        resolver = AccessServer.getResolverPlugin();

        logger.info("TcpServer configured on port <" + port + ">.");
    }

    /** The main method of the thread. It is called by the Java runtime environment.
     * This method waits for new connections and creates an instance of the inner class
     * TcpHandler for each new one.
     */    
    public void run()
    {
        try
        {
            logger.info("Starting TcpServer on port " + port);
            listen = new ServerSocket(port);
            
            while (running)
            {
                TcpHandler tcpHandler = new TcpHandler(listen.accept());
                handlers.add(tcpHandler);
            }
        }
        catch (IOException e)
        {
            if (running)
                logger.error("Socket error on port " + port, e);
        }
    }

    /** This method is called by the AccessServer when the background threads should
     * stop because of a shutdown of the AccessServer itself.
     */
    public void shutdown()
    {
        running = false;
        
        try
        {
            listen.close();
        }
        catch (IOException e)
        {
            // Do nothing
        }

        interrupt();
        
        Enumeration enum = handlers.elements();
        
        while (enum.hasMoreElements())
            ((TcpHandler) enum.nextElement()).shutdown();
            
    }
    
    /** Implements the handler for each established connection. */    
    public class TcpHandler extends Thread
    {
        /** CVS Version Tag */
        private static final String vcid = "$Id: TcpServer.java,v 1.14 2003/04/14 20:41:41 joerg Exp $";
    
        private Socket socket = null;
    
        private Logger logger = Logger.getLogger(this.getClass());

        private ExpressionEvaluator evaluator = new ExpressionEvaluator();

        private Pattern pCommandLine;
        
        private String user = null;
    
        private boolean localHostFlag = false;
        
        private boolean running = true;
        
        private boolean ignoreCase = false;
        
        private TcpHandler()
        {
        }
    
        /** Creates a new instance of TcpHandler
         * @param s The socket to use for the communication with the client.
         */
        public TcpHandler(Socket s)
        {
            socket = s;

            setName("TcpHandler-" + socket.getPort());

            start();
        }

        /** This method is called by the AccessServer when the background threads should
         * stop because of a shutdown of the AccessServer itself.
         */        
        protected void shutdown()
        {
            running = false;

            try
            {
                socket.close();
            }
            catch (IOException e)
            {
                // Do nothing
            }
            
            interrupt();
        }
        
        private boolean processLine(PrintStream os, String commandLine)
        {
            boolean result = true;
            
            Matcher mCommandLine = pCommandLine.matcher(commandLine);
                                        
            if (! mCommandLine.find())
            {
                os.println("ERROR 100: Unknown command");
            }
            else
            {
                int count = mCommandLine.groupCount();

                if (count == 3)
                {
                    String cmd      = mCommandLine.group(1).trim();
                    String operator = mCommandLine.group(2);
                    String argument = mCommandLine.group(3).trim();
                           
                    if (ignoreCase)
                        argument = argument.toLowerCase();

                    if (operator.equals("") && argument.equals(""))
                    {
                        if (cmd.equalsIgnoreCase("HELP"))
                        {                            
                            os.println("HELP - This help");
                            os.println("QUIT - Close connection");
                            os.println("SETUSER=<user> - Set user for expressions");
                            os.println("EXPR=<expression> - Evaluate expression");
                            os.println("OK");
                        }
                        else if (cmd.equalsIgnoreCase("QUIT"))
                        {
                            os.println("Closing connection");
                            result = false;
                        }
                        else if (localHostFlag && cmd.equalsIgnoreCase("SHUTDOWN"))
                        {
                            os.println("Closing connection and shutting down");
                            logger.info("Shutdown command recognized");
                            AccessServer.shutDown();
                            result = false;
                        }
                        else    
                        {
                            os.println("ERROR 100: Unknown command");
                        }
                    }
                    else if (! operator.equals("="))
                    {
                        os.println("ERROR 101: Syntax error");
                    }
                    else if (operator.equals("=") && argument.equals(""))
                    {
                        os.println("ERROR 101: Syntax error");
                    }
                    else if (cmd.equalsIgnoreCase("SETUSER") && argument.indexOf(" ") == -1)
                    {
                        boolean flag = true;

                        if (verifyUser)
                        {
                            long t = System.currentTimeMillis();
                            flag = resolver.verifyUser(argument);
                            t = System.currentTimeMillis() - t;

                            logger.info("User <" + argument + "> verified, ellapsed Time = " + t + "ms");
                        }

                        if (flag)
                        {
                            user = argument;

                            logger.info("Setting user = <" + argument + ">");
                            os.println("OK");
                        }
                        else
                        {
                            user = null;
                            logger.error("Invalid user ID = <" + argument + ">");
                            os.println("ERROR 202: Invalid user ID");
                        }
                    }
                    else if (cmd.equalsIgnoreCase("EXPR"))
                    {
                        if (user == null)
                        {
                            logger.error("No user set");
                            os.println("ERROR 200: No user set");
                        }
                        else
                        {
                            logger.info("Evaluating <" + argument + 
                                "> for user <" + user + ">");

                            long t = System.currentTimeMillis();

                            try 
                            {
                                boolean answer = evaluator.evaluateExpression(user, argument);
                                    
                                t = System.currentTimeMillis() - t;

                                os.println(answer);
                                os.println("OK");
                                    
                                logger.info("Answer = <" + answer + 
                                    ">, ellapsed Time = " + t + "ms");
                            }
                            catch (ParserException pe)
                            {
                                logger.error("Error while parsing expression <" + argument + ">", pe);
                                os.println("ERROR 201: Expression <" + argument + "> can not be evaluated - " + pe.getMessage());
                            }
                        }
                    }
                    else
                    {
                        os.println("ERROR 100: Unknown command");
                    }
                }
                else
                {
                    os.println("ERROR 101: Syntax error");
                }
            }
            
            return result;
        }
        
        /** This is the main loop of the server. It receives and dispatches the commands
         * and sends the results back to the client.
         */        
        public void run()
        {
            ignoreCase = AccessServer.getIgnoreCase();
            
            pCommandLine = Pattern.compile("^\\s*([a-zA-Z0-9]+)\\s*(=?)\\s*(.*)\\s*$");

            InetAddress inetAdr = socket.getInetAddress();
            String ipAddr = inetAdr.getHostAddress();
            int port = socket.getPort();
            logger.info("Connection opened from " + ipAddr + 
                    " (" + inetAdr.getCanonicalHostName() + ") port " + port);

            if (ipAddr.equals("0:0:0:0:0:0:0:1") || ipAddr.equals("127.0.0.1"))
            {
                logger.info("Connection from localhost recognized");
                localHostFlag = true;
            }
            
            try
            {            
                BufferedReader is = new BufferedReader(new InputStreamReader(socket.getInputStream()));
                PrintStream os = new PrintStream(socket.getOutputStream());

                os.println("iiitAccessServer - For help send command \"HELP\"");
                os.println("OK");
            
                while (running)
                {
                    String request = is.readLine();
                
                    if (request == null)
                        running = false;
                    else
                        running = running && processLine(os, request);
                }
                
                logger.info("Connection to " + ipAddr + " port " + port + " closed.");

                is.close();
                os.close();
                socket.close();
            }
            catch (IOException e)
            {
                if (running)
                    logger.error("IO exception caught while listening to " + ipAddr + " port " + port, e);
                
                try
                {
                    socket.close();
                }
                catch (IOException ie)
                {
                    // Do nothing
                }                    
            }
            
            handlers.remove(this);
        }
    }
}

/**
 * $Log: TcpServer.java,v $
 * Revision 1.14  2003/04/14 20:41:41  joerg
 * Setting name for thread.
 *
 * Revision 1.13  2003/04/13 21:09:56  joerg
 * Package structure modified
 *
 * Revision 1.12  2003/04/13 20:28:01  joerg
 * Package structure modified
 *
 * Revision 1.11  2003/04/13 20:16:42  joerg
 * Package structure modified
 *
 * Revision 1.10  2003/03/27 20:39:22  joerg
 * Names of constants changed.
 *
 * Revision 1.9  2003/01/17 19:57:48  joerg
 * Wenn die globale Config-Variable VerifyUser den Wert TRUE
 * hat, wird beim Kommando setuser die UserId ueberprueft.
 *
 * Revision 1.8  2003/01/04 17:15:43  joerg
 * Zustzliche Config-Option IgnoreCase
 *
 * Revision 1.7  2003/01/02 16:31:17  joerg
 * Fehlerkorrektur
 *
 * Revision 1.6  2003/01/01 21:16:55  joerg
 * Fehlermeldungen um eindeutige Nummern erweitert.
 *
 * Revision 1.5  2003/01/01 21:04:18  joerg
 * Copyright-Statement aktualisiert
 *
 * Revision 1.4  2002/12/27 20:50:23  joerg
 * Fehlerbehandlung beim Oeffnen des Sockets
 *
 * Revision 1.3  2002/12/22 21:02:10  joerg
 * Fehler beim shutdown() behoben
 *
 * Revision 1.2  2002/12/22 20:45:23  joerg
 * shutdown()-Methode eingebaut
 *
 * Revision 1.1  2002/12/19 15:24:23  joerg
 * Reparatur des CVS-Repositories
 *
 * Revision 1.17  2002/12/09 21:14:15  joerg
 * AccessServerThreadPluginIf umbenannt in ThreadPluginIf
 *
 * Revision 1.16  2002/12/09 19:29:17  joerg
 * Versionsdaten in allen JavaDoc-Klassenbeschreibungen ergaenzt
 *
 * Revision 1.15  2002/12/09 10:35:09  joerg
 * Javadoc vervollstaendigt
 * DefaultPort ist jetzt 54321
 *
 * Revision 1.14  2002/12/08 14:35:01  joerg
 * Plugin-Interface neu strukturiert
 *
 * Revision 1.13  2002/11/30 21:19:46  joerg
 * Kommando-Verarbeitung komplett ueberarbeitet
 *
 * Revision 1.12  2002/11/27 22:30:34  joerg
 * Fehlermeldung verbessert
 *
 * Revision 1.11  2002/11/26 14:57:14  joerg
 * Fehlerbehandlung korrigiert.
 *
 * Revision 1.10  2002/11/26 10:56:06  joerg
 * Package exprparser durch parser erstzt.
 *
 * Revision 1.9  2002/11/21 08:34:23  joerg
 * Ausgaben an den Client ueberarbeitet
 *
 * Revision 1.8  2002/11/20 11:10:38  joerg
 * Umgebaut zumServerPlugin
 * ParserExceptions werden behandelt
 *
 * Revision 1.7  2002/11/06 11:15:47  joerg
 * Jede Instanz hat jetzt ihren eigenen ExpressionEvaluator
 *
 * Revision 1.6  2002/11/04 09:38:09  joerg
 * Klasse ExpressionResolver ins Package AccessServer verschoben
 * und in ExpressionEvaluator umbenannt.
 *
 * Revision 1.5  2002/11/03 21:56:27  joerg
 * Aufloesung von Ausdruecken implementiert.
 *
 * Revision 1.4  2002/11/03 20:39:08  joerg
 * Log-Ausgaben an neuen Klassenname angepasst.
 *
 * Revision 1.3  2002/11/03 20:31:46  joerg
 * Klasse TcpListener umbenannt in TcpServer
 *
 * Revision 1.2  2002/11/03 20:28:36  joerg
 * Klasse TcpServer als InnerClass TcpHandler nach TcpListener verschoben.
 *
 * Revision 1.1  2002/11/03 14:45:03  joerg
 * Klassen neu angelegt
 *
 */
